"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = transformActionGroup;
const utils_1 = require("../../shared/utils");
const transforms_1 = require("../../shared/transforms");
const getComponents_1 = require("../../../getComponents");
const t = __importStar(require("@babel/types"));
let availableComponents = (0, getComponents_1.getComponents)();
/**
 * Transforms ActionGroup:
 * - Comment out overflowMode (it has not been implemented yet).
 * - Comment out buttonLabelBehavior (it has not been implemented yet).
 * - Comment out summaryIcon (it has not been implemented yet).
 * - Use ActionButtonGroup if no selection.
 * - Use ToggleButtonGroup if selection is used.
 * - Update root level onAction to onPress on each ActionButton.
 * - Apply isDisabled directly on each ActionButton/ToggleButton instead of disabledKeys.
 * - Convert dynamic collections render function to items.map.
 */
function transformActionGroup(path) {
    // Comment out overflowMode
    (0, transforms_1.commentOutProp)(path, { propName: 'overflowMode' });
    // Comment out buttonLabelBehavior
    (0, transforms_1.commentOutProp)(path, { propName: 'buttonLabelBehavior' });
    // Comment out summaryIcon
    (0, transforms_1.commentOutProp)(path, { propName: 'summaryIcon' });
    let selectionModePath = path.get('openingElement').get('attributes').find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'selectionMode');
    let selectionMode = t.isStringLiteral(selectionModePath?.node.value) ? selectionModePath.node.value.value : 'none';
    let newComponentName, childComponentName;
    if (selectionMode === 'none') {
        // Use ActionButtonGroup if no selection
        newComponentName = 'ActionButtonGroup';
        childComponentName = 'ActionButton';
        selectionModePath?.remove();
    }
    else {
        // Use ToggleButtonGroup if selection is used
        newComponentName = 'ToggleButtonGroup';
        childComponentName = 'ToggleButton';
    }
    let localName = newComponentName;
    if (availableComponents.has(newComponentName)) {
        let program = path.findParent((p) => t.isProgram(p.node));
        localName = (0, utils_1.addComponentImport)(program, newComponentName);
    }
    let localChildName = childComponentName;
    if (availableComponents.has(childComponentName)) {
        let program = path.findParent((p) => t.isProgram(p.node));
        localChildName = (0, utils_1.addComponentImport)(program, childComponentName);
    }
    // Convert dynamic collection to an array.map.
    let items = path.get('openingElement').get('attributes').find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'items');
    let itemArg;
    if (items && t.isJSXExpressionContainer(items.node.value) && t.isExpression(items.node.value.expression)) {
        let child = path.get('children').find(c => c.isJSXExpressionContainer());
        if (child && child.isJSXExpressionContainer() && t.isFunction(child.node.expression)) {
            let arg = child.node.expression.params[0];
            if (t.isIdentifier(arg)) {
                itemArg = arg;
            }
            child.replaceWith(t.jsxExpressionContainer(t.callExpression(t.memberExpression(items.node.value.expression, t.identifier('map')), [child.node.expression])));
        }
    }
    items?.remove();
    let onAction = path.get('openingElement').get('attributes').find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'onAction');
    // Pull disabledKeys prop out into a variable, converted to a Set.
    // Then we can check it in the isDisabled prop of each item.
    let disabledKeysPath = path.get('openingElement').get('attributes').find((attr) => t.isJSXAttribute(attr.node) && attr.node.name.name === 'disabledKeys');
    let disabledKeys;
    if (disabledKeysPath && t.isJSXExpressionContainer(disabledKeysPath.node.value) && t.isExpression(disabledKeysPath.node.value.expression)) {
        disabledKeys = path.scope.generateUidIdentifier('disabledKeys');
        path.scope.push({
            id: disabledKeys,
            init: t.newExpression(t.identifier('Set'), [disabledKeysPath.node.value.expression]),
            kind: 'let'
        });
        disabledKeysPath.remove();
    }
    path.traverse({
        JSXElement(child) {
            if (t.isJSXIdentifier(child.node.openingElement.name) && child.node.openingElement.name.name === 'Item') {
                // Replace Item with ActionButton or ToggleButton.
                let childNode = t.cloneNode(child.node);
                childNode.openingElement.name = t.jsxIdentifier(localChildName);
                if (childNode.closingElement) {
                    childNode.closingElement.name = t.jsxIdentifier(localChildName);
                }
                // If there is no key prop and we are using dynamic collections, add a default computed from item.key ?? item.id.
                let key = childNode.openingElement.attributes.find(attr => t.isJSXAttribute(attr) && attr.name.name === 'key');
                if (!key && itemArg) {
                    let id = t.jsxExpressionContainer(t.logicalExpression('??', t.memberExpression(itemArg, t.identifier('key')), t.memberExpression(itemArg, t.identifier('id'))));
                    key = t.jsxAttribute(t.jsxIdentifier('key'), id);
                    childNode.openingElement.attributes.push(key);
                }
                // If this is a ToggleButtonGroup, add an id prop in addition to key when needed.
                if (key && newComponentName === 'ToggleButtonGroup') {
                    // If we are in an array.map we need both key and id. Otherwise, we only need id.
                    if (itemArg) {
                        childNode.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('id'), key.value));
                    }
                    else {
                        key.name.name = 'id';
                    }
                }
                let keyValue = undefined;
                if (key && t.isJSXExpressionContainer(key.value) && t.isExpression(key.value.expression)) {
                    keyValue = key.value.expression;
                }
                else if (key && t.isStringLiteral(key.value)) {
                    keyValue = key.value;
                }
                // Add an onPress to each item that calls the previous onAction, passing in the key.
                if (onAction && t.isJSXExpressionContainer(onAction.node.value) && t.isExpression(onAction.node.value.expression)) {
                    childNode.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('onPress'), t.jsxExpressionContainer(keyValue
                        ? t.arrowFunctionExpression([], t.callExpression(onAction.node.value.expression, [keyValue]))
                        : onAction.node.value.expression)));
                }
                // Add an isDisabled prop to each item, testing whether it is in disabledKeys.
                if (disabledKeys && keyValue) {
                    childNode.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('isDisabled'), t.jsxExpressionContainer(t.callExpression(t.memberExpression(disabledKeys, t.identifier('has')), [keyValue]))));
                }
                child.replaceWith(childNode);
            }
        }
    });
    onAction?.remove();
    path.node.openingElement.name = t.jsxIdentifier(localName);
    if (path.node.closingElement) {
        path.node.closingElement.name = t.jsxIdentifier(localName);
    }
}
