"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = transformComboBox;
const transforms_1 = require("../../shared/transforms");
/**
 * Transforms ComboBox:
 * - Change menuWidth value from a DimensionValue to a pixel value.
 * - Remove isQuiet (it is no longer supported in Spectrum 2).
 * - Comment out loadingState (it has not been implemented yet).
 * - Remove placeholder (it is no longer supported in Spectrum 2).
 * - Change validationState="invalid" to isInvalid.
 * - Remove validationState="valid" (it is no longer supported in Spectrum 2).
 * - Comment out onLoadMore (it has not been implemented yet).
 */
function transformComboBox(path) {
    // Change menuWidth value from a DimensionValue to a pixel value
    (0, transforms_1.convertDimensionValueToPx)(path, { propName: 'menuWidth' });
    // Remove isQuiet
    (0, transforms_1.removeProp)(path, { propName: 'isQuiet' });
    // Comment out loadingState
    (0, transforms_1.commentOutProp)(path, { propName: 'loadingState' });
    // Remove placeholder
    (0, transforms_1.removeProp)(path, { propName: 'placeholder' });
    // Change validationState="invalid" to isInvalid
    (0, transforms_1.updatePropNameAndValue)(path, {
        oldPropName: 'validationState',
        oldPropValue: 'invalid',
        newPropName: 'isInvalid',
        newPropValue: true
    });
    // Remove validationState="valid"
    (0, transforms_1.removeProp)(path, { propName: 'validationState', propValue: 'valid' });
    // Comment out onLoadMore
    (0, transforms_1.commentOutProp)(path, { propName: 'onLoadMore' });
}
